@@ -6,8 +6,9 @@ import os
 import platform
 import subprocess
 import locale
+import cairo
 gi.require_version("Gtk", "3.0")
-from gi.repository import Gtk
+from gi.repository import Gtk, Gio, Gdk, GdkPixbuf
 
 NORUN_FLAG = os.path.expanduser("~/.linuxmint/mintwelcome/norun.flag")
 
@@ -17,6 +18,8 @@ from locale import gettext as _
 locale.bindtextdomain("mintwelcome", "/usr/share/linuxmint/locale")
 locale.textdomain("mintwelcome")
 
+LAYOUT_STYLE_LEGACY, LAYOUT_STYLE_NEW = range(2)
+
 class SidebarRow(Gtk.ListBoxRow):
 
     def __init__(self, page_widget, page_name, icon_name):
@@ -82,10 +85,15 @@ class MintWelcome():
         builder.get_object("button_mintinstall").connect("clicked", self.launch, "mintinstall")
         builder.get_object("button_timeshift").connect("clicked", self.pkexec, "timeshift-gtk")
         builder.get_object("button_mintdrivers").connect("clicked", self.pkexec, "driver-manager")
+        builder.get_object("button_gufw").connect("clicked", self.launch, "gufw")
+        builder.get_object("button_layout_legacy").connect("clicked", self.on_button_layout_clicked, LAYOUT_STYLE_LEGACY)
+        builder.get_object("button_layout_new").connect("clicked", self.on_button_layout_clicked, LAYOUT_STYLE_NEW)
 
         # Settings button depends on DE
+        de_is_cinnamon = False
         if os.getenv("XDG_CURRENT_DESKTOP") in ["Cinnamon", "X-Cinnamon"]:
             builder.get_object("button_settings").connect("clicked", self.launch, "cinnamon-settings")
+            de_is_cinnamon = True
         elif os.getenv("XDG_CURRENT_DESKTOP") == "MATE":
             builder.get_object("button_settings").connect("clicked", self.launch, "mate-control-center")
         elif os.getenv("XDG_CURRENT_DESKTOP") == "XFCE":
@@ -94,6 +102,10 @@ class MintWelcome():
             # Hide settings
             builder.get_object("box_first_steps").remove(builder.get_object("box_settings"))
 
+        # Hide Cinnamon layout settings in other DEs
+        if not de_is_cinnamon:
+            builder.get_object("box_first_steps").remove(builder.get_object("box_cinnamon"))
+
         # Hide codecs box if they're already installed
         add_codecs = False
         cache = apt.Cache()
@@ -147,9 +159,28 @@ class MintWelcome():
         checkbox.connect("toggled", self.on_button_toggled)
         box.pack_end(checkbox)
 
+        scale = window.get_scale_factor()
+
+        # Use HIDPI pictures if appropriate
+        if scale == 1:
+            surface = self.surface_for_path("/usr/share/linuxmint/mintwelcome/legacy.png", scale)
+            builder.get_object("img_legacy").set_from_surface(surface)
+            surface = self.surface_for_path("/usr/share/linuxmint/mintwelcome/modern.png", scale)
+            builder.get_object("img_modern").set_from_surface(surface)
+        else:
+            surface = self.surface_for_path("/usr/share/linuxmint/mintwelcome/legacy-hidpi.png", scale)
+            builder.get_object("img_legacy").set_from_surface(surface)
+            surface = self.surface_for_path("/usr/share/linuxmint/mintwelcome/modern-hidpi.png", scale)
+            builder.get_object("img_modern").set_from_surface(surface)
+
         window.set_default_size(800, 500)
         window.show_all()
 
+    def surface_for_path(self, path, scale):
+        pixbuf = GdkPixbuf.Pixbuf.new_from_file(path)
+
+        return Gdk.cairo_surface_create_from_pixbuf(pixbuf, scale)
+
     def sidebar_row_selected_cb(self, list_box, row):
         self.stack.set_visible_child(row.page_widget)
 
@@ -161,6 +192,65 @@ class MintWelcome():
             os.system("mkdir -p ~/.linuxmint/mintwelcome")
             os.system("touch %s" % NORUN_FLAG)
 
+    def on_button_layout_clicked (self, button, style):
+
+        applets_legacy = ['panel1:left:0:menu@cinnamon.org',
+                          'panel1:left:1:show-desktop@cinnamon.org',
+                          'panel1:left:2:panel-launchers@cinnamon.org',
+                          'panel1:left:3:window-list@cinnamon.org',
+                          'panel1:right:0:systray@cinnamon.org',
+                          'panel1:right:1:keyboard@cinnamon.org',
+                          'panel1:right:2:notifications@cinnamon.org',
+                          'panel1:right:3:removable-drives@cinnamon.org',
+                          'panel1:right:4:user@cinnamon.org',
+                          'panel1:right:5:network@cinnamon.org',
+                          'panel1:right:6:sound@cinnamon.org',
+                          'panel1:right:7:power@cinnamon.org',
+                          'panel1:right:8:calendar@cinnamon.org']
+
+        applets_new = ['panel1:left:0:menu@cinnamon.org',
+                       'panel1:left:1:show-desktop@cinnamon.org',
+                       'panel1:left:2:grouped-window-list@cinnamon.org',
+                       'panel1:right:0:systray@cinnamon.org',
+                       'panel1:right:1:keyboard@cinnamon.org',
+                       'panel1:right:2:notifications@cinnamon.org',
+                       'panel1:right:3:removable-drives@cinnamon.org',
+                       'panel1:right:4:network@cinnamon.org',
+                       'panel1:right:5:sound@cinnamon.org',
+                       'panel1:right:6:power@cinnamon.org',
+                       'panel1:right:7:calendar@cinnamon.org']
+
+        settings = Gio.Settings("org.cinnamon")
+        settings.set_strv("panels-enabled", ['1:0:bottom'])
+
+        applets = applets_new
+        left_icon_size = 0
+        center_icon_size = 0
+        right_icon_size = 0
+        theme = "Mint-Y"
+        cinnamon_theme = "Mint-Y-Dark"
+        if style == LAYOUT_STYLE_LEGACY:
+            applets = applets_legacy
+            panel_size = 27
+            theme = "Mint-X"
+            cinnamon_theme = "Linux Mint"
+            menu_label = "Menu"
+        elif style == LAYOUT_STYLE_NEW:
+            panel_size = 40
+            right_icon_size = 24
+            menu_label = ""
+
+        settings.set_strv("panels-height", ['1:%s' % panel_size])
+        settings.set_strv("enabled-applets", applets)
+        settings.set_string("app-menu-label", menu_label)
+        settings.set_string("panel-zone-icon-sizes", "[{\"panelId\": 1, \"left\": %s, \"center\": %s, \"right\": %s}]" % (left_icon_size, center_icon_size, right_icon_size))
+
+        Gio.Settings('org.cinnamon.desktop.wm.preferences').set_string('theme', theme)
+        Gio.Settings('org.cinnamon.theme').set_string('name', cinnamon_theme)
+        Gio.Settings('org.cinnamon.desktop.interface').set_string('icon-theme', theme)
+        Gio.Settings('org.cinnamon.desktop.interface').set_string('gtk-theme', theme)
+        os.system("cinnamon --replace &")
+
     def visit(self, button, url):
         subprocess.Popen(["xdg-open", url])
 
